

// A simple digital clock program using YACL

#include "ui/ui.h"
#include "base/base.h"


// The main window:

class ClockWindow: public UI_CompositeVObject {
public:
    ClockWindow ();

    bool Update (CL_Object&, long);
    void Initialize ();

protected:
    UI_Label*               _lbl;
    CL_Binding<ClockWindow> _bind;
    UI_Timer                _timer;
};



typedef CL_Binding <ClockWindow> Bind;

#if defined(__GNUC__)
template class CL_Binding <ClockWindow>; // Instantiate the template
#endif


ClockWindow::ClockWindow ()
: UI_CompositeVObject (NULL, NULL, FALSE, UI_Rectangle (50, 50, 200, 90)),
  _bind  (this, (Bind::MethodPtr) &ClockWindow::Update),
  _timer (_bind)
{
    _lbl = new UI_Label (this, UI_Rectangle (50, 30, 100, 30));
}

void ClockWindow::Initialize()
{
    UI_CompositeVObject::Initialize();
    _lbl->Title() = CL_TimeOfDay::Now().AsString();
    _timer.Start (1000); // Call Update every 1000 millseconds
}

bool ClockWindow::Update (CL_Object&, long)
{
    _lbl->Title() = CL_TimeOfDay::Now().AsString();
    _Controller->DispatchPendingEvents (); // Flush the event queue so that
                                           // the update shows on the screen
    return TRUE;
}



// The main program:

int UI_Application::Main (int, char* [])
{
    UI_CompositeVObject* root = new ClockWindow;
    Controller().MakeTopWindow (root);
    root->Title()  = "YACL clock";
    Run();
    return 0;
}

